﻿using System;


// This adaptable state machine is a bit more complex than the simple one presented earlier.

// The biggest change is that extensible code follows the "Dependancy Inversion" principle.
// That means that instead of implementing states and machines directly, I have created
// two classes called IState and IStateMachine.  These two classes declare the interface
// that all state machines will follow.

// By creating a state machine interface I can later extend the system to create many
// different types of state machines.  But more on that later.

// After I carefully considered what goes in my state machine and what belongs in my state
// (that part took about 10 minutes even though I have written thousands of state machines
// over my career) I then created most of this file.  I could create everything except the
// "CreateNewMachine" method.  During development it just returned null.

// After getting the main class worked out, I then created instances of the IState and
// IStateMachine classes.


namespace AdaptableStateMachine
{
    class Program
    {
        static void Main(string[] args)
        {
            // First we need to create the state machine.
            // Note that I'm using the abstract IStateMachine instead of a concrete class.
            IStateMachine machine = GetMachine();

            // We have a machine, now run it.
            while(!machine.IsComplete())
            {
                // Print out our current state
                Console.WriteLine("Currently in " + machine.CurrentState);
                machine.CurrentState.Run();

                // Print out our possible transitions
                Console.WriteLine("\nAvailable choices are:");
                string[] transitions = machine.PossibleTransitions();
                foreach (string item in transitions)
                {
                    Console.WriteLine("  " + item);
                }

                // Request a transition from the user
                Console.WriteLine("\nWhat do you want to do?");
                string nextState = Console.ReadLine();
                machine.Advance(nextState);                              
            }

            // And we're done!
            // Run our final node as a special case since the above loop won't do it.
            Console.WriteLine("Currently in " + machine.CurrentState);
            machine.CurrentState.Run();

            // Finish off.
            Console.WriteLine("\n\nPress any key to continue.");
            Console.ReadKey(true);
        }
        private static IStateMachine GetMachine()
        {
            IStateMachine machine = null;

            while (machine == null)
            {
                Console.WriteLine("Choose your state machine:");
                Console.WriteLine(" 1. Boring state machine.");
                Console.WriteLine(" 2. Fun state machine.");
                Console.WriteLine(" 3. Saved state machine.");

                ConsoleKeyInfo key = Console.ReadKey();

                switch (key.Key)
                {
                    case ConsoleKey.NumPad1:
                    case ConsoleKey.D1:
                        machine = new BoringStateMachine.BoringMachine();
                        break;
                    case ConsoleKey.NumPad2:
                    case ConsoleKey.D2:
                        machine = new FunStateMachine.FunMachine();
                        break;
                    case ConsoleKey.NumPad3:
                    case ConsoleKey.D3:
                        machine = new SavedStateMachine.SavedMachine();
                        break;
                    default:
                        Console.WriteLine("\nUnknown State Machine!\n");
                        break;
                }
            }

            return machine;
        }
    }
}
