﻿using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using System.Xml;

namespace AdaptableStateMachine.SavedStateMachine
{
    public class SavedMachine : IStateMachine, IXmlSerializable
    {
        #region Members
        List<SavedMachineState> mStates = new List<SavedMachineState>();
        SavedMachineState mCurrent;
        #endregion
        #region Constructor
        /// <summary>
        /// Initializes a new instance of the FunnerMachine class.
        /// </summary>
        public SavedMachine()
        {
            try
            {
                ImportFromXML();
            }
            catch (Exception ex)
            {
                mStates.Clear();
            }

            if (mStates.Count == 0)
            {
                GenerateDefaultMap();
                ImportFromXML();
            }

            // Find the entry state
            for (int i = 0; i < mStates.Count; i++)
            {
                if (mStates[i].IsStartState)
                {
                    mCurrent = mStates[i];
                    break;
                }
            }
            if (mCurrent == null)
            {
                Console.WriteLine("\n\nERROR! NO ENTRY STATE DEFINED.");
                throw new Exception("No entry state defined in this state machine.  Cannot continue.");
            }
        }
        #endregion
        #region Helper Functions
        private void GenerateDefaultMap()
        {
            mStates.Clear();

            // Create all the fun states in our mini-world
            mStates.Add(new SavedMachineState("entryHall", SavedMachineState.StateFlags.Enter, "Grand Entrance", "You are standing in a grand enterance of a castle.\nThere are tables and chairs, but nothing you can interact with.", "staircase,outside"));
            mStates.Add(new SavedMachineState("staircase", SavedMachineState.StateFlags.None, "Grand Staircase", "The staircase is made from beautiful granite.", "eastWing,westWing,entryHall"));
            mStates.Add(new SavedMachineState("eastWing", SavedMachineState.StateFlags.None, "East Wing", "This wing is devoted to bedrooms.", "bedroomA,bedroomB,bedroomC,staircase"));
            mStates.Add(new SavedMachineState("westWing", SavedMachineState.StateFlags.None, "West Wing", "This wing is devoted to business.", "workroomA,workroomB,workroomC"));
            mStates.Add(new SavedMachineState("bedroomA", SavedMachineState.StateFlags.None, "Master Suite", "This is the master suite.  What a fancy room.", "eastWing"));
            mStates.Add(new SavedMachineState("bedroomB", SavedMachineState.StateFlags.None, "Prince Bob's Room", "The prince has an extensive library on his wall.\nHe also has more clothes than most males know what to do with.", "eastWing"));
            mStates.Add(new SavedMachineState("bedroomC", SavedMachineState.StateFlags.None, "Princess Alice's Room", "The princess has filled her room with a small compur lab.\nShe spends her days playing games and writing code.", "eastWing"));
            mStates.Add(new SavedMachineState("workroomA", SavedMachineState.StateFlags.None, "Study", "This is the study.  It has many books.", "westWing"));
            mStates.Add(new SavedMachineState("workroomB", SavedMachineState.StateFlags.None, "Bathroom", "Every home needs one", "westWing"));
            mStates.Add(new SavedMachineState("workroomC", SavedMachineState.StateFlags.None, "Do Not Enter", "I warned you not to enter.\nYou are in a maze of twisty little passages, all alike.", "passage"));
            mStates.Add(new SavedMachineState("passage", SavedMachineState.StateFlags.None, "Twisty Passage", "You are in a maze of twisty little passages, all alike", "passage"));
            mStates.Add(new SavedMachineState("outside", SavedMachineState.StateFlags.Exit, "Outside", "You have successfully exited the castle.", ""));

            ExportToXML();
        }
        public void ExportToXML()
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;
            settings.OmitXmlDeclaration = true;
            settings.NewLineHandling = NewLineHandling.Entitize;

            using (XmlWriter writer = XmlWriter.Create("GameRooms.xml",settings))
            {
                writer.WriteStartDocument();
                writer.WriteStartElement("SavedMachine");
                WriteXml(writer);
                writer.WriteEndElement();
                writer.WriteEndDocument();
            }           
        }
        public void ImportFromXML()
        {
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.IgnoreWhitespace = true;
            XmlReader reader = XmlReader.Create("GameRooms.xml", settings);
            ReadXml(reader);
        }
        #endregion
        #region IStateMachine Overrides
        public override IState CurrentState
        {
            get { return mCurrent; }
        }
        public override string[] PossibleTransitions()
        {
            List<string> result = new List<string>();
            foreach (string state in mCurrent.Neighbors)
            {
                result.Add(state);
            }
            return result.ToArray();
        }
        public override bool Advance(string nextState)
        {
            foreach (SavedMachineState state in mStates)
            {
                if(state.IsMyName(nextState)
                    && mCurrent.Neighbors.Contains(state.Key))
                {
                    mCurrent = state;
                    return true;
                }
            }
            System.Console.WriteLine("Cannot do that.");
            return false;
        }
        public override bool IsComplete()
        {
            return mCurrent.IsExitState;
        }
        #endregion
        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }


        public void ReadXml(XmlReader reader)
        {
            bool isEmpty = reader.IsEmptyElement;
            reader.ReadStartElement();
            if (isEmpty) return;
            while (reader.NodeType == XmlNodeType.Element)
            {
                if (reader.Name == "Room")
                {
                    mStates.Add(new SavedMachineState(reader));
                }
                else
                    throw new XmlException("Unexpected node: " + reader.Name);
            }
            reader.ReadEndElement();
        }

        public void WriteXml(XmlWriter writer)
        {
            foreach (SavedMachineState state in mStates)
            {
                writer.WriteStartElement("Room");
                state.WriteXml(writer);
                writer.WriteEndElement();
            }
        }

        #endregion
    }
}
