﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Xml;

namespace AdaptableStateMachine.SavedStateMachine
{
    public class SavedMachineState : IState, IXmlSerializable
    {
        #region Members
        [Flags]
        public enum StateFlags
        {
            None = 0,
            Enter = 1,
            Exit = 2,
        }

        public string mKey;
        StateFlags mFlags;
        string mName;
        string mDescription;
        List<string> mNeighbors = new List<string>();
        public List<string> Neighbors { get { return mNeighbors; } }
        #endregion
        #region Constructors
        /// <summary>
        /// Manual constructor for default maze
        /// </summary>
        /// <param name="uniqueKey">unique name for the stateFlags</param>
        /// <param name="flags">flags to indicate enter nodes and exit nodes</param>
        /// <param name="name">name to show to the user</param>
        /// <param name="description">text to show for the description</param>
        /// <param name="neighbors">unique keys for neighboring rooms, seperated by commas and not spaces</param>
        public SavedMachineState(string uniqueKey, StateFlags flags, string name, string description, string neighbors)
        {
            mKey = uniqueKey;
            mFlags = flags;
            mName = name;
            mDescription = description;
            mNeighbors.AddRange(neighbors.Split(','));
        }
        /// <summary>
        /// Constructor to create an object from a save file
        /// </summary>
        /// <param name="reader">xml stream to read from</param>
        public SavedMachineState(XmlReader reader)
        {
            ReadXml(reader);
        }
        #endregion
        #region Helper Functions
        public bool IsStartState { get { return (mFlags & StateFlags.Enter) != StateFlags.None; } }
        public bool IsExitState { get { return (mFlags & StateFlags.Exit) != StateFlags.None; } }
        public string Key { get { return mKey; } }
        public bool IsMyName(string nameToTest)
        {
            //TODO: Add shortcuts to names.  For example, allow "Great Hall", "Hall", etc.
            if (nameToTest.ToLower() == mName.ToLower())
                return true;
            if (nameToTest.ToLower() == mKey.ToLower())
                return true;
            return false;
        }
        #endregion
        #region IState Overrides
        public override string GetName()
        {
            return mName;
        }

        public override void Run()
        {
            // We don't do any fancy stuff, just print out where we are
            Console.WriteLine();
            Console.WriteLine(mDescription);
        }
        #endregion
        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.ReadStartElement();
            mKey = reader.ReadElementContentAsString("UniqueName","");
            string flagString = reader.ReadElementContentAsString("Flags","");
            mFlags = (StateFlags)Enum.Parse(typeof(StateFlags), flagString);
            mName = reader.ReadElementContentAsString("VisibleName", "");
            mDescription = reader.ReadElementContentAsString("Description", "");
            string neighborsString = reader.ReadElementContentAsString("Neighbors", "");
            mNeighbors.AddRange(neighborsString.Split(','));
            reader.ReadEndElement();
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            writer.WriteElementString("UniqueName", mKey);
            writer.WriteElementString("Flags", mFlags.ToString());
            writer.WriteElementString("VisibleName", mName);
            writer.WriteElementString("Description", mDescription);
            string neighbors = String.Join(",",Neighbors.ToArray());
            writer.WriteElementString("Neighbors",neighbors);
        }

        #endregion
    }
}
